unit CenterForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  MCenterForm = class(TComponent)
  private
    { Private declarations }
    fOwner:TForm;
    fOrHeight:Integer;
    fOrWidth:Integer;
    fOldTWndMethod:TWndMethod;
    fAllowMove : boolean;
    fAllowSize : boolean;
  protected
    { Protected declarations }
    procedure WndProc(var Message: TMessage);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property AllowMoving : boolean read fAllowMove write fAllowMove;
    property AllowSizing : boolean read fAllowSize write fAllowSize;
  end;

procedure Register;

implementation

constructor MCenterForm.Create(AOwner: TComponent);
begin
   If AOwner is TForm then begin
      inherited Create(AOwner);
      fOwner:=TForm(AOwner);
      fOldTWndMethod:=fOwner.WindowProc;
      fOwner.WindowProc:=WndProc;
      fAllowMove:=false;
      fAllowSize:=true;
      fOrHeight:=fOwner.Height;
      fOrWidth:=fOwner.Width;
   end
   else begin
      ShowMessage('Owner of this component must be a Form');
      exit;
   end;
end;

destructor MCenterForm.Destroy;
begin
   fOwner.WindowProc:=fOldTWndMethod;
   inherited Destroy;
end;

procedure MCenterForm.WndProc(var Message: TMessage);
var mLeft,mTop:integer;
begin
  if (CsDesigning in ComponentState) then fOldTwndMethod(Message) else
  case Message.Msg of
  WM_MOVE   : begin
              if fAllowMove then begin
                 fOldTwndMethod(Message);
              end
              else begin
                 mLeft := (Screen.Width - fOwner.Width) div 2;
                 mTop := (Screen.Height - fOwner.Height) div 2;
                 fOwner.SetBounds(mLeft, mTop, fOwner.Width, fOwner.Height);
              end;
              end;
  WM_SIZE   : begin
              if fAllowSize then begin
                 fOldTwndMethod(Message);
                 if not(fAllowMove) then begin
                    mLeft := (Screen.Width - fOwner.Width) div 2;
                    mTop := (Screen.Height - fOwner.Height) div 2;
                    fOwner.SetBounds(mLeft, mTop, fOwner.Width, fOwner.Height);
                 end;
              end else begin
                 if not(fAllowMove) then begin
                    mLeft := (Screen.Width - fOrWidth) div 2;
                    mTop := (Screen.Height - fOrHeight) div 2;
                    fOwner.SetBounds(mLeft, mTop, fOrWidth, fOrHeight);
                 end else begin
                    mLeft := fOwner.Left;
                    mTop := fOwner.Top;
                    fOwner.SetBounds(mLeft, mTop, fOrWidth, fOrHeight);
                 end;
              end;
              end
  else fOldTwndMethod(Message);
  end; {case}
end;

procedure Register;
begin
  RegisterComponents('Milka', [MCenterForm]);
end;

end.
